import configparser
import datetime
import logging
import sys
import time
import uuid

from carom import worker

try:
    import RPi.GPIO as GPIO
except ImportError:
    print("please install RPi.GPIO Module")

logging.basicConfig(stream=sys.stdout, level=logging.INFO)
log = logging.getLogger(__name__)

# chan_io = {38: 1, 36: 2, 32: 3, 26: 4, 24: 5, 22: 6, 18: 7, 16: 8}
chan_io = dict()
last_state = dict()


class CaromClient:
    @staticmethod
    def get_uuid():
        """
        read uuid from raspi mac address

        :return: the generated host uuid
        """
        return uuid.uuid5(uuid.NAMESPACE_DNS, str(uuid.getnode()))

    @staticmethod
    def get_gpio_status():
        """
        print out gpio status on console

        :return: gpio result
        """
        result = ''
        for key, value in chan_io.items():
            result += " {0}: {1} ".format(value, key)
        return result

    def __init__(self, args):
        """
        init carom client class

        :param args: cmd line args
        """
        self.config = configparser.ConfigParser()
        self.args = args
        self.init_config()

    def init_config(self):
        """
        init configuration
        """
        self.config.read(self.args.c)
        self.init_gpio_config()

    def init_gpio_config(self):
        for pin in range(1, 9):
            chan_io[int(self.config['mapping'][str(pin)])] = int(pin)

    def init_gpio(self):
        GPIO.setmode(GPIO.BOARD)
        GPIO.setup(list(chan_io.keys()), GPIO.IN, pull_up_down=GPIO.PUD_DOWN)
        self.runner()
        log.info('init ready')

    def runner(self):
        """
        runner for directly read gpio states

        last_state 0: switch is disabled
        last_state 1: switch is first time enabled. Next state is 2
        last_state 2: switch is enabled
        last_state 3: switch is first time disabled

        :return:
        """
        while True:
            for key, pin in chan_io.items():
                state = GPIO.input(key)
                if self.args.v:
                    log.info('Processing: {0}, {1}, {2}'.format(str(pin), str(key), str(state)))
                if key not in last_state:
                    last_state[key] = state
                last = last_state[key]
                if last == 0:
                    if state == 1:
                        log.info("Status change: desk_no: {0}: new state: {1}".format(pin, state))
                        last_state[key] = 1
                elif last == 1:
                    if state == 1:
                        self.send_to_server(state, pin)
                        last_state[key] = 2
                    if state == 0:
                        last_state[key] = 0
                elif last == 2:
                    if state == 0:
                        log.info("Status change: desk_no: {0}: new state: {1}".format(pin, state))
                        last_state[key] = 3
                elif last == 3:
                    if state == 0:
                        self.send_to_server(state, pin)
                        last_state[key] = 0
                    if state == 1:
                        last_state[key] = 2
            time.sleep(2)
        pass

    def get_io_pin(self, chan):
        return chan_io.get(chan)

    def cleanup(self):
        GPIO.cleanup()
        log.info('cleanup done')

    def send_to_server(self, state, pin):
        d = datetime.datetime.now()
        url = self.config['connection']['endpoint'].format('billard/api/v1/locationdata/')
        headers = {'Authorization': 'Token {}'.format(self.config['connection']['token']),
                   'Content-Type': 'application/json'}
        payload = {'client_id': str(CaromClient.get_uuid()),
                   'desk_no': str(pin),
                   'tst': d.strftime('%Y-%m-%dT%H:%M:%S%Z'),
                   'on_off': state}
        log.info("Send Status: {}, {}, {}".format(str(url), str(headers), str(payload)))
        worker.send_to_server(url=url, payload=payload, headers=headers)

    def start(self):
        try:
            self.init_gpio()
            while True:
                if self.args.v and log.isEnabledFor(logging.DEBUG):
                    for chan in list(chan_io.keys()):
                        log.debug("IO: {0}: {1}".format(self.get_io_pin(chan), GPIO.input(chan)))
                time.sleep(5)
        except KeyboardInterrupt:
            log.info('keyboard interrupted')
        except Exception as e:
            log.exception(e)
        finally:
            self.cleanup()
